/*    
    Copyright (C) Paul Falstad and Iain Sharp
    Modified by Vinyasi 25/Sep/2018 19:38

// Mod.Begin
// Mod.End
    
    This file is part of CircuitJS1.

    CircuitJS1 is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    CircuitJS1 is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with CircuitJS1.  If not, see <http://www.gnu.org/licenses/>.

	================================= Vinnie's Theory on making Coil Inductance more Realistic =================================

	Convert from inductance to equivalent series resistance to be more realistic, 
	ie eliminate a super conducting coil based on a few assumptions...

	[article on Wikipedia]
	http://is.gd/eqseres

	I use standard charts depicting wire resistance, weight and length per AWG, and then I multiply by a factor of 0.003125 per 
	Henry. This factor is derived from the motor coil of Joseph Newman described by Dr. Hastings in chapter six of Newman's book, 
	any edition, entitled: "The Energy Machine of Joseph Newman". Since one tenth AWG is a base ten logarythm, I divide AWG by ten, 
	raise ten to that power, and then multiply by 0.003125 times inductance to get the equivalent series resistance for that coil.

	Consequently, and for example, a coil of 30 AWG wound to exhibit 50,000 Ohms of resistance and 16,000 Henrys of inductance
	is also exhibiting a ratio of a little over three Ohms per Henry.

	Rendered into Java software code...

	double wireGauge = 20;
	double wireGaugeConstant = 0.003125;
	double seriesResistance;

	seriesResistance = Math.pow(10, (wireGauge / 10) ) * wireGaugeConstant * inductance;

	References...
	AWG Copper Wire Size Table and Data Chart @ 100 Degrees F _ Engineers Edge
	at...
	engineersedge.com

	https://is.gd/navalu
	https://www.superioressex.com/uploadedFiles/Magnet_Wire_and_Distribution/North_America/Magnet_Wire_-_Winding_Wire/EngData_book_linked.pdf

	Now that I've upgraded this software, many if not most of my prior circuits may not work due to a loss of resonance among their 
	various components requiring some tweaking to get them going again, or not.

*/

package com.lushprojects.circuitjs1.client;

class Inductor {
    public static final int FLAG_BACK_EULER = 2;
    int nodes[];
    int flags;
    CirSim sim;
    double inductance;
// Mod.Begin
	double wireGauge;
	double wireGaugeConstant;
	double seriesResistance;
// Mod.End
    double compResistance, current;
    double curSourceValue;
    Inductor(CirSim s) {
	sim = s;
	nodes = new int[2];
    }
// Mod.Begin
//    void setupRelay(double ic, double cr, int f) {
    void setupRelay(double ic, double cr) {
	inductance = ic;
	current = cr;
//	flags = f;
    }
//    void setup(double ic, double cr, int f, double wg) {
    void setup(double ic, double cr, double wg) {
// Mod.End
		inductance = ic;
// Mod.Begin
		wireGaugeConstant = 0.003125;
		wireGauge = wg;
		seriesResistance = Math.pow(10, (wireGauge / 10) ) * wireGaugeConstant * inductance;
// Mod.End
		current = cr;
//		flags = f;
    }
// Mod.Begin
//    boolean isTrapezoidal() { return (flags & FLAG_BACK_EULER) == 0; }
// Mod.End
    void reset() {
	// need to set curSourceValue here in case one of inductor nodes is node 0.  In that case
	// calculateCurrent() may get called (from setNodeVoltage()) when analyzing circuit, before
	// startIteration() gets called
	current = curSourceValue = 0;
    }
    void stamp(int n0, int n1) {
	// inductor companion model using trapezoidal or backward euler
	// approximations (Norton equivalent) consists of a current
	// source in parallel with a resistor.  Trapezoidal is more
	// accurate than backward euler but can cause oscillatory behavior.
	// The oscillation is a real problem in circuits with switches.
	nodes[0] = n0;
	nodes[1] = n1;
// Mod.Begin
//	if (isTrapezoidal())
//	    compResistance = 2 * inductance / sim.timeStep;
//	else // backward euler
//	    compResistance = inductance / sim.timeStep;
//	    compResistance = seriesResistance * 2 * inductance / sim.timeStep;
//	} else { // backward euler
	    compResistance = seriesResistance * inductance / sim.timeStep;
//	}
// Mod.End
	sim.stampResistor(nodes[0], nodes[1], compResistance);
	sim.stampRightSide(nodes[0]);
	sim.stampRightSide(nodes[1]);
    }
    boolean nonLinear() { return false; }

    void startIteration(double voltdiff) {
// Mod.Begin
//		if (isTrapezoidal()) {
// Mod.End
//			curSourceValue = voltdiff/compResistance+current;
//		} else { // backward euler
	    	curSourceValue = current;
//		}
	}
    
    double calculateCurrent(double voltdiff) {
	// we check compResistance because this might get called
	// before stamp(), which sets compResistance, causing
	// infinite current
	if (compResistance > 0)
	    current = voltdiff / compResistance + curSourceValue;
	return current;
    }
    void doStep(double voltdiff) {
	sim.stampCurrentSource(nodes[0], nodes[1], curSourceValue);
    }
}
